/* Copyright (c) 2009 Nordic Semiconductor. All Rights Reserved.
 *
 * The information contained herein is confidential property of Nordic
 * Semiconductor ASA.Terms and conditions of usage are described in detail
 * in NORDIC SEMICONDUCTOR STANDARD SOFTWARE LICENSE AGREEMENT.
 *
 * Licensees are granted free, non-transferable use of the information. NO
 * WARRANTY of ANY KIND is provided. This heading must NOT be removed from
 * the file.
 *
 */

/** @file
 * @brief RTC/PPI/ADC modules example
 *
 * This example enables the RTC with a TICK frequency of 10Hz. Each RTC TICK triggers
 * ADC conversion through a PPI channel.
 * @image html example_board_setup_a.png "Use board setup A for this example."
 */

#include <stdbool.h>
#include "nrf.h"
#include "nrf_delay.h"
#include "nrf_gpio.h"

#define GPIO_TOGGLE_TICK_EVENT    (18)                       /*!< Pin number to toggle when there is a tick event in RTC */
#define GPIO_TOGGLE_COMPARE_EVENT (19) /*!< Pin number to toggle when there is compare event in RTC */
#define LFCLK_FREQUENCY           (32768UL)                 /*!< LFCLK frequency in Hertz, constant */
#define RTC_FREQUENCY             (10UL)                     /*!< required RTC working clock RTC_FREQUENCY Hertz. Changable */
#define COUNTER_PRESCALER         ((LFCLK_FREQUENCY/RTC_FREQUENCY) - 1)  /* f = LFCLK/(prescaler + 1) */

#define COMPARE_COUNTERTIME       (2UL)                     /*!< Get Compare event COMPARE_TIME seconds after the counter starts from 0 */

#define RC_RTC_CAL_1_SEC          (4UL)
#define RC_RTC_CAL_INTERVAL       (4 * RC_RTC_CAL_1_SEC)

/** Configures port 1 as output
 */
static void gpio_config(void)
{
    nrf_gpio_range_cfg_output(GPIO_TOGGLE_TICK_EVENT, GPIO_TOGGLE_COMPARE_EVENT);
}

static void lfclk_config(void)
{
    NRF_CLOCK->LFCLKSRC = (CLOCK_LFCLKSRC_SRC_RC << CLOCK_LFCLKSRC_SRC_Pos);
    NRF_CLOCK->EVENTS_LFCLKSTARTED = 0;
    NRF_CLOCK->TASKS_LFCLKSTART = 1;

    // Wait for the low frequency clock to start
    while (NRF_CLOCK->EVENTS_LFCLKSTARTED == 0) {}
    NRF_CLOCK->EVENTS_LFCLKSTARTED = 0;
}

static void hfclk_config(void)
{
    /* Start 16 MHz crystal oscillator */
    NRF_CLOCK->EVENTS_HFCLKSTARTED = 0;
    NRF_CLOCK->TASKS_HFCLKSTART = 1;

    /* Wait for the external oscillator to start up */
    while (NRF_CLOCK->EVENTS_HFCLKSTARTED == 0) 
    {
    }
}

static void init_rc_osc_cal_timer(void)
{
    NRF_CLOCK->CTIV = RC_RTC_CAL_INTERVAL;
    NRF_CLOCK->INTENSET = CLOCK_INTENSET_CTTO_Msk;
    NVIC_EnableIRQ(POWER_CLOCK_IRQn);
    
    NRF_CLOCK->TASKS_CTSTART = 1;
    
}

/** Configure and start RTC
 */
static void rtc_config(void)
{
    NVIC_EnableIRQ(RTC0_IRQn);
    NRF_RTC0->CC[0] = COMPARE_COUNTERTIME * RTC_FREQUENCY;
    NRF_RTC0->PRESCALER = COUNTER_PRESCALER;                   // Set prescaler to a TICK of RTC_FREQUENCY
    // Enable TICK event and TICK interrupt:
    NRF_RTC0->EVTENSET = RTC_EVTENSET_TICK_Msk;
    NRF_RTC0->INTENSET = RTC_INTENSET_TICK_Msk;

    // Enable COMPARE0 event and COMPARE0 interrupt:
    NRF_RTC0->EVTENSET = RTC_EVTENSET_COMPARE0_Msk;
    NRF_RTC0->INTENSET = RTC_INTENSET_COMPARE0_Msk;
    NRF_RTC0->TASKS_START = 1;                                 // Start RTC0
}

void RTC0_IRQHandler(void)
{
    if (NRF_RTC0->EVENTS_TICK)
    {
        NRF_RTC0->EVENTS_TICK = 0;
        nrf_gpio_pin_toggle(GPIO_TOGGLE_TICK_EVENT);
    }
    // This will trigger every COMPARE_COUNTERTIME seconds
    if (NRF_RTC0->EVENTS_COMPARE[0])
    {
        #warning "you can alternatively set the RC OSC calibration here"
        
        static uint32_t compare_data;
        NRF_RTC0->EVENTS_COMPARE[0] = 0;        
        
        // Reload compare
        compare_data += (COMPARE_COUNTERTIME * RTC_FREQUENCY);        
        NRF_RTC0->CC[0] = (COMPARE_COUNTERTIME * RTC_FREQUENCY) + compare_data;
        
        nrf_gpio_pin_toggle(GPIO_TOGGLE_COMPARE_EVENT);
    }
}

void POWER_CLOCK_IRQHandler(void)
{
    if (NRF_CLOCK->EVENTS_CTTO)
    {
        NRF_CLOCK->EVENTS_CTTO = 0;
        
        // Wait for 16M XOSC
        while (NRF_CLOCK->EVENTS_HFCLKSTARTED == 0);        
        // Do Calibration  
               
        NRF_CLOCK->TASKS_CAL = 1;
        while (NRF_CLOCK->EVENTS_DONE == 0)
        {
        }
        // Restart timer
        NRF_CLOCK->TASKS_CTSTART = 1;
        NRF_CLOCK->EVENTS_DONE = 0;         
    }
}

/**
 * main function
 */
int main(void)
{	
    gpio_config();
    lfclk_config();
    hfclk_config();
    init_rc_osc_cal_timer();
    rtc_config();
	
    while (true)
    {	
        //Put CPU to sleep while waiting for interrupt
        __WFI();
    }
}

/**
 *@}
 **/
